@@ -24,6 +24,10 @@ class ArmModule(envi.ArchitectureModule):
         self.badoplist = [self.archParseOpcode(badop, 0, 0) for badop in self._arch_badopbytes]
         self.badoplist.extend([self.archParseOpcode(badop, 0, 1) for badop in self._arch_badopbytes])
 
+    def initRegGroups(self):
+        envi.ArchitectureModule.initRegGroups(self)
+        self._regGrps.update({'general': arm_regs})
+
     def archGetRegCtx(self):
         return ArmRegisterContext()
 
@@ -42,7 +46,7 @@ class ArmModule(envi.ArchitectureModule):
     def pointerString(self, va):
         return "0x%.8x" % va
 
-    def archParseOpcode(self, bytes, offset=0, va=0):
+    def archParseOpcode(self, bytes, offset=0, va=0, extra=None):
         """
         Parse a sequence of bytes out into an envi.Opcode instance.
         """
@@ -72,17 +76,6 @@ class ArmModule(envi.ArchitectureModule):
             return tova & -2, reftype, rflags
         return tova, reftype, rflags
 
-    def archGetRegisterGroups(self):
-        groups = envi.ArchitectureModule.archGetRegisterGroups(self)
-
-        groups.append(('general', arm_regs))
-
-        # compilers use the following regs to stick the module baseaddr in for 
-        # switchcase code
-        switch_mapbase = ('switch_mapbase', [ 'r0', 'r1', 'r2', 'r3',],)   # FIXME: limited sampleset.  this list could be longer
-        groups.append(switch_mapbase)
-        return groups
-
     def archGetPointerAlignment(self):
         return 4
 
@@ -105,6 +98,10 @@ class ThumbModule(envi.ArchitectureModule):
         self.badoplist = [ self.archParseOpcode(badop,0,0) for badop in self._arch_badopbytes ]
         self.badoplist.extend([ self.archParseOpcode(badop,0,1) for badop in self._arch_badopbytes ])
 
+    def initRegGroups(self):
+        envi.ArchitectureModule.initRegGroups(self)
+        self._regGrps.update({'general': arm_regs})
+
     def archGetRegCtx(self):
         return ArmRegisterContext()
 
@@ -123,7 +120,7 @@ class ThumbModule(envi.ArchitectureModule):
     def pointerString(self, va):
         return "0x%.8x" % va
 
-    def archParseOpcode(self, bytes, offset=0, va=0):
+    def archParseOpcode(self, bytes, offset=0, va=0, extra=None):
         """
         Parse a sequence of bytes out into an envi.Opcode instance.
         """
