@@ -167,7 +167,11 @@ class VQMemoryCanvas(e_memcanvas.MemoryCanvas, QWebEngineView):
         self._canv_cache = self._canv_cache.replace('`', r'\`')
         js = f'''
         var node = document.querySelector('update#updatetmp');
-        node.outerHTML = `{self._canv_cache}` + node.outerHTML;
+        if (node != null) {{
+            node.outerHTML = `{self._canv_cache}` + node.outerHTML;
+        }} else {{
+            console.log("Failed to grab selector of update#updatetmp")
+        }}
         '''
         if cb:
             self.page().runJavaScript(js, cb)
@@ -320,6 +324,8 @@ def getNavTargetNames():
 
 
 def initMemSendtoMenu(expr, menu):
-    for name in set(getNavTargetNames()):
+    navtgts = getNavTargetNames()
+    navtgts.sort()
+    for name in navtgts:
         args = (name, expr, None)
         menu.addAction('sendto: %s' % name, ACT(vqtevent, 'envi:nav:expr', args))
